@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -35,6 +35,7 @@
 #include <goffice/graph/gog-theme.h>
 #include <goffice/graph/gog-error-bar.h>
 #include <goffice/graph/gog-reg-eqn.h>
+#include <goffice/graph/gog-series-lines.h>
 #include <goffice/data/go-data-simple.h>
 #include <goffice/utils/format.h>
 #include <goffice/utils/go-font.h>
@@ -115,6 +116,7 @@ libgoffice_init (void)
 	(void) GOG_GRID_LINE_TYPE;
 	(void) GOG_ERROR_BAR_TYPE;
 	(void) GOG_REG_EQN_TYPE;
+	(void) GOG_SERIES_LINES_TYPE;
 	(void) GO_DATA_SCALAR_VAL_TYPE;
 	(void) GO_DATA_SCALAR_STR_TYPE;
 	gog_themes_init	();
