@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -218,8 +218,9 @@ map_discrete_calc_ticks (GogAxis *axis)
 	GogAxisTick *ticks;
 	gboolean valid;
 	double maximum, minimum;
-	int tick_nbr;
-	int i;
+	double tick_start, label_start;
+	int tick_nbr, label_nbr;
+	int i, j;
 	int major_tick, major_label;
 
 	major_tick = rint (gog_axis_get_entry (axis, GOG_AXIS_ELEM_MAJOR_TICK, NULL));
@@ -235,36 +236,39 @@ map_discrete_calc_ticks (GogAxis *axis)
 		return;
 	}
 		
-	tick_nbr = rint (maximum -minimum) + 1;
-	if (tick_nbr < 1 || tick_nbr > GOG_AXIS_MAX_TICK_NBR) {
-		gog_axis_set_ticks (axis, 0, NULL);
+	tick_start = axis->center_on_ticks ? 
+		ceil (minimum / (double) major_tick) * major_tick : 
+		ceil ((minimum - 0.5) / (double) major_tick) * major_tick + 0.5;
+	label_start = ceil (minimum / (double) major_label) * major_label;
+	tick_nbr = floor (go_add_epsilon (maximum - tick_start) / major_tick + 1.0);
+	label_nbr = floor (go_add_epsilon (maximum - label_start) / major_label + 1.0);
+	tick_nbr = CLAMP (tick_nbr, 0, GOG_AXIS_MAX_TICK_NBR);
+	label_nbr = CLAMP (label_nbr, 0, GOG_AXIS_MAX_TICK_NBR);
+	if (tick_nbr < 1  && label_nbr < 1) {
+		gog_axis_set_ticks (axis, 2, create_invalid_axis_ticks (0.0, 1.0));
 		return;
 	}
-	ticks = g_new (GogAxisTick, tick_nbr);
+	ticks = g_new (GogAxisTick, tick_nbr + label_nbr);
 	
 	for (i = 0; i < tick_nbr; i++) {
-		ticks[i].position = (double) (i);
-
-		ticks[i].type = i % major_tick == 0 ?
-			GOG_AXIS_TICK_MAJOR :
-			GOG_AXIS_TICK_MINOR;
-
-		if ((i % major_label == 0) && 
-		    (i < tick_nbr - 1 || axis->center_on_ticks)) {
-			if (axis->labels != NULL) {
-				if (i < go_data_vector_get_len (axis->labels))
-					ticks[i].label = go_data_vector_get_str (axis->labels, i);
-				else
-					ticks[i].label = NULL;
-			}
-			else
-				ticks[i].label = g_strdup_printf ("%d", i + 1);
-		} else
-			    ticks[i].label = NULL;
-
+		ticks[i].position = tick_start + (double) (i) * major_tick;
+		ticks[i].type = GOG_AXIS_TICK_MAJOR;
+		ticks[i].label = NULL;
 	}
-	
-	gog_axis_set_ticks (axis, tick_nbr, ticks);
+	for (i = 0, j = tick_nbr; i < label_nbr; i++, j++) {
+		ticks[j].position = label_start + (double) (i) * major_label;
+		ticks[j].type = GOG_AXIS_TICK_NONE;
+		if (axis->labels != NULL) {
+			if (i * major_label < go_data_vector_get_len (axis->labels))
+				ticks[j].label = go_data_vector_get_str (axis->labels, i * major_label);
+			else
+				ticks[j].label = NULL;
+		}
+		else
+			ticks[j].label = g_strdup_printf ("%d", i * major_label + 1);
+	}
+
+	gog_axis_set_ticks (axis, tick_nbr + label_nbr, ticks);
 }
 
 /*
@@ -339,8 +343,8 @@ map_bounds (GogAxisMap *map, double *minimum, double *maximum)
 {
 	MapData *data = map->data;
 
-	*minimum = data->min;
-	*maximum = data->max;	
+	if (minimum != NULL) *minimum = data->min;
+	if (maximum != NULL) *maximum = data->max;	
 }
 
 static void
@@ -560,8 +564,8 @@ map_log_bounds (GogAxisMap *map, double *minimum, double *maximum)
 {
 	MapLogData *data = map->data;
 
-	*minimum = exp (data->min);
-	*maximum = exp (data->max);	
+	if (minimum != NULL) *minimum = exp (data->min);
+	if (maximum != NULL) *maximum = exp (data->max);	
 }
 
 static void
@@ -861,6 +865,8 @@ gog_axis_map_get_baseline (GogAxisMap *map)
  * axis is not inverted, start = minimum and stop = maximum. If axis is invalid, 
  * it'll return arbitrary bounds. For example, an non inverted invalid X axis 
  * will have start set to 0.0 and stop set to 1.0.
+ *
+ * minimum or maximum can be NULL.
  * */
 
 void
@@ -880,7 +886,9 @@ gog_axis_map_get_extents (GogAxisMap *map, double *start, double *stop)
  *
  * Returns bounds for the given axis map in data coordinates. If axis is invalid, 
  * it'll return arbitrary bounds. For example, for an invalid x axis, minimum = 0.0
- * and maximum = 1.0.
+ * and maximum = 1.0. 
+ *
+ * minimum or maximum can be NULL.
  * */
 
 void
@@ -989,21 +997,6 @@ enum {
 
 /*****************************************************************************/
 
-static gboolean
-role_grid_line_can_add (GogObject const *parent, char const *type)
-{
-	GSList *children;
-
-	children = gog_object_get_children (parent, 
-		gog_object_find_role_by_name (GOG_OBJECT (parent), type));
-	if (children != NULL) {
-		g_slist_free (children);
-		return FALSE;
-	}
-
-	return TRUE;
-}
-
 static gboolean
 role_grid_line_major_can_add (GogObject const *parent)
 {
@@ -1012,7 +1005,7 @@ role_grid_line_major_can_add (GogObject const *parent)
 	
 	return ((type == GOG_AXIS_X || type == GOG_AXIS_Y || type == GOG_AXIS_RADIAL || 
 		 (type == GOG_AXIS_CIRCULAR && !gog_axis_is_discrete (axis))) &&
-		role_grid_line_can_add (parent, "MajorGrid"));
+		gog_axis_get_grid_line (GOG_AXIS (parent), TRUE) == NULL);
 }
 
 static gboolean
@@ -1024,7 +1017,7 @@ role_grid_line_minor_can_add (GogObject const *parent)
 	return (!gog_axis_is_discrete (GOG_AXIS (parent)) &&
 		(type == GOG_AXIS_X || type == GOG_AXIS_Y || 
 		 type == GOG_AXIS_RADIAL || type == GOG_AXIS_CIRCULAR) &&
-		role_grid_line_can_add (parent, "MinorGrid"));
+		gog_axis_get_grid_line (GOG_AXIS (parent), FALSE) == NULL);
 }
 
 static void 
@@ -1059,6 +1052,15 @@ role_axis_line_post_add (GogObject *parent, GogObject *child)
 	gog_axis_base_set_position (GOG_AXIS_BASE (child), GOG_AXIS_AUTO);
 }
 
+static gboolean 
+role_label_can_add (GogObject const *parent)
+{
+	GogAxisType type = gog_axis_get_atype (GOG_AXIS (parent));
+	
+	return (type == GOG_AXIS_X ||
+		type == GOG_AXIS_Y);
+}
+
 /**
  * gog_axis_set_format :
  * @axis : #GogAxis
@@ -1274,8 +1276,8 @@ gog_axis_update (GogObject *obj)
 			g_object_ref (labels);
 			axis->labels = GO_DATA_VECTOR (labels);
 			axis->plot_that_supplied_labels = GOG_PLOT (ptr->data);
-			axis->center_on_ticks = bounds.center_on_ticks;
 		}
+		axis->center_on_ticks = bounds.center_on_ticks;
 
 		if (axis->min_val > bounds.val.minima) {
 			axis->min_val = bounds.val.minima;
@@ -1498,7 +1500,7 @@ gog_axis_populate_editor (GogObject *gobj,
 	(GOG_OBJECT_CLASS(parent_klass)->populate_editor) (gobj, editor, dalloc, cc);
 
 	/* Format page */
-	if (!axis->is_discrete) {
+	if (!axis->is_discrete && gog_axis_get_atype (axis) != GOG_AXIS_PSEUDO_3D) {
 		w = go_format_sel_new ();
 		if (axis->assigned_format != NULL && !style_format_is_general (axis->assigned_format))
 			go_format_sel_set_style_format (GO_FORMAT_SEL (w),
@@ -1536,27 +1538,31 @@ gog_axis_populate_editor (GogObject *gobj,
 static void
 gog_axis_init_style (GogStyledObject *gso, GogStyle *style)
 {
-	style->interesting_fields = GOG_STYLE_LINE | GOG_STYLE_FONT;
+	if (gog_axis_get_atype (GOG_AXIS (gso)) != GOG_AXIS_PSEUDO_3D)
+		style->interesting_fields = GOG_STYLE_LINE | GOG_STYLE_FONT |
+			GOG_STYLE_TEXT_LAYOUT;
+	else
+		style->interesting_fields = 0;
 	gog_theme_fillin_style (gog_object_get_theme (GOG_OBJECT (gso)),
-		style, GOG_OBJECT (gso), 0, FALSE);
+				style, GOG_OBJECT (gso), 0, FALSE);
 }
 
 static void
 gog_axis_class_init (GObjectClass *gobject_klass)
 {
 	static GogObjectRole const roles[] = { 
-		{ N_("Label"), "GogLabel", 0,
-		  GOG_POSITION_SPECIAL, GOG_POSITION_SPECIAL, GOG_OBJECT_NAME_BY_ROLE,
-		  NULL, NULL, NULL, NULL, NULL, NULL, { -1 } },
-		{ N_("MinorGrid"), "GogGridLine", 2,
-		  GOG_POSITION_SPECIAL, GOG_POSITION_SPECIAL, GOG_OBJECT_NAME_BY_ROLE,
-		  role_grid_line_minor_can_add, NULL, NULL, role_grid_line_minor_post_add, NULL, NULL, { -1 } },
-		{ N_("MajorGrid"), "GogGridLine", 1,
+		{ N_("MajorGrid"), "GogGridLine", 0,
 		  GOG_POSITION_SPECIAL, GOG_POSITION_SPECIAL, GOG_OBJECT_NAME_BY_ROLE,
 		  role_grid_line_major_can_add, NULL, NULL, role_grid_line_major_post_add, NULL, NULL, { -1 } },
-		{ N_("AxisLine"), "GogAxisLine", 0,
+		{ N_("MinorGrid"), "GogGridLine", 1,
+		  GOG_POSITION_SPECIAL, GOG_POSITION_SPECIAL, GOG_OBJECT_NAME_BY_ROLE,
+		  role_grid_line_minor_can_add, NULL, NULL, role_grid_line_minor_post_add, NULL, NULL, { -1 } },
+		{ N_("AxisLine"), "GogAxisLine", 2,
 		  GOG_POSITION_PADDING, GOG_POSITION_PADDING, GOG_OBJECT_NAME_BY_ROLE,
-		  role_axis_line_can_add, NULL, NULL, role_axis_line_post_add, NULL, NULL, { -1 } }
+		  role_axis_line_can_add, NULL, NULL, role_axis_line_post_add, NULL, NULL, { -1 } },
+		{ N_("Label"), "GogLabel", 3,
+		  GOG_POSITION_SPECIAL|GOG_POSITION_ANY_MANUAL, GOG_POSITION_SPECIAL, GOG_OBJECT_NAME_BY_ROLE,
+		  role_label_can_add, NULL, NULL, NULL, NULL, NULL, { -1 } }
 	};
 
 	GogObjectClass *gog_klass = (GogObjectClass *) gobject_klass;
@@ -1651,8 +1657,8 @@ gog_axis_dataset_init (GogDatasetClass *iface)
 }
 
 GSF_CLASS_FULL (GogAxis, gog_axis,
-		gog_axis_class_init, gog_axis_init,
-		GOG_AXIS_BASE_TYPE, 0,
+		NULL, NULL, gog_axis_class_init, NULL,
+		gog_axis_init, GOG_AXIS_BASE_TYPE, 0,
 		GSF_INTERFACE (gog_axis_dataset_init, GOG_DATASET_TYPE))
 
 
@@ -1847,6 +1853,30 @@ gog_axis_bound_changed (GogAxis *axis, GogObject *contrib)
 	gog_object_request_update (GOG_OBJECT (axis));
 }
 
+/* gog_axis_get_grid_line:
+ * @axis: #GogAxis
+ * @major: wether to retrieve major or minor grid line.
+ *
+ * Returns a pointer to GridLine object associated to given axis, NULL
+ * if it doesn't exists.
+ **/
+GogGridLine *
+gog_axis_get_grid_line (GogAxis *axis, gboolean major)
+{
+	GogGridLine *grid_line;
+	GSList *children;
+
+	children = gog_object_get_children (GOG_OBJECT (axis), 
+		gog_object_find_role_by_name (GOG_OBJECT (axis), 
+			major ? "MajorGrid" : "MinorGrid"));
+	if (children != NULL) {
+		grid_line = GOG_GRID_LINE (children->data);
+		g_slist_free (children);
+		return grid_line;
+	}
+	return NULL;
+}
+
 /****************************************************************************/
 
 typedef GogAxisBaseView		GogAxisView;
@@ -1879,15 +1909,15 @@ gog_axis_view_padding_request (GogView *view,
 	for (ptr = view->children; ptr != NULL ; ptr = ptr->next) {
 		child = ptr->data;
 		pos = child->model->position;
-		if (IS_GOG_LABEL (child->model) && (pos & GOG_POSITION_SPECIAL)) {
+		if (IS_GOG_LABEL (child->model) && !(pos & GOG_POSITION_MANUAL)) {
 			gog_view_size_request (child, &req);
 			if (type == GOG_AXIS_X) 
 				label_padding.hb += req.h + pad_h;
 			else  
 				label_padding.wl += req.w + pad_w;
-	}
 		}
-		
+	}
+
 	tmp.x += label_padding.wl;
 	tmp.w -= label_padding.wl + label_padding.wr;
 	tmp.y += label_padding.hb;
@@ -1897,14 +1927,14 @@ gog_axis_view_padding_request (GogView *view,
 
 	for (ptr = view->children; ptr != NULL ; ptr = ptr->next) {
 		child = ptr->data;
-		if (child->model->position == GOG_POSITION_PADDING) {
+		if (GOG_POSITION_IS_PADDING (child->model->position)) {
 			gog_view_padding_request (child, &tmp, &child_padding);
 			padding->wr = MAX (padding->wr, child_padding.wr);
 			padding->wl = MAX (padding->wl, child_padding.wl);
 			padding->hb = MAX (padding->hb, child_padding.hb);
 			padding->ht = MAX (padding->ht, child_padding.ht);
-}
-}
+		}
+	}
 
 	padding->wr += label_padding.wr;
 	padding->wl += label_padding.wl;
@@ -1930,26 +1960,34 @@ gog_axis_view_size_allocate (GogView *view, GogViewAllocation const *bbox)
 	for (ptr = view->children; ptr != NULL ; ptr = ptr->next) {
 		child = ptr->data;
 		pos = child->model->position;
-		if (pos & GOG_POSITION_SPECIAL) {
-			if (IS_GOG_LABEL (child->model)) {
-				gog_view_size_request (child, &req);
-				if (type == GOG_AXIS_X) {
-					child_bbox.x = plot_area->x + (plot_area->w - req.w) / 2.0;
-					child_bbox.w = plot_area->w;
-					child_bbox.y = tmp.y + tmp.h - req.h;
-					child_bbox.h = req.h;
-					tmp.h -= req.h + pad_h;
+		if (IS_GOG_LABEL (child->model) && (pos & GOG_POSITION_MANUAL)) {
+			gog_view_size_request (child, &req);
+			child_bbox = gog_object_get_manual_allocation (gog_view_get_model (child), 
+								       plot_area, &req);
+			gog_view_size_allocate (child, &child_bbox);
 		} else {
-					child_bbox.x = tmp.x;
-					child_bbox.w = req.w;
-					child_bbox.y = plot_area->y + (plot_area->h - req.h) / 2.0;
-					child_bbox.h = plot_area->h;
-					tmp.x += req.w + pad_w;
-					tmp.w -= req.w + pad_w;
+			if (GOG_POSITION_IS_SPECIAL (pos)) {
+				if (IS_GOG_LABEL (child->model)) {
+					gog_view_size_request (child, &req);
+					if (type == GOG_AXIS_X) {
+						child_bbox.x = plot_area->x + (plot_area->w - req.w) / 2.0;
+						child_bbox.w = plot_area->w;
+						child_bbox.y = tmp.y + tmp.h - req.h;
+						child_bbox.h = req.h;
+						tmp.h -= req.h + pad_h;
+					} else {
+						child_bbox.x = tmp.x;
+						child_bbox.w = req.w;
+						child_bbox.y = plot_area->y + (plot_area->h - req.h) / 2.0;
+						child_bbox.h = plot_area->h;
+						tmp.x += req.w + pad_w;
+						tmp.w -= req.w + pad_w;
+					}
+					gog_view_size_allocate (child, &child_bbox);
+				} else {
+					gog_view_size_allocate (child, plot_area);
+				}
 			}
-				gog_view_size_allocate (child, &child_bbox);
-			} else 
-				gog_view_size_allocate (child, plot_area);
 		}	
 	}
 }
