@@ -15,11 +15,12 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
 #include <goffice/goffice-config.h>
+#include <goffice/graph/gog-axis.h>
 #include <goffice/graph/gog-label.h>
 #include <goffice/graph/gog-outlined-object.h>
 #include <goffice/graph/gog-style.h>
@@ -29,17 +30,19 @@
 #include <goffice/graph/gog-data-set.h>
 #include <goffice/graph/gog-data-allocator.h>
 #include <goffice/data/go-data.h>
-
-#include <glib/gi18n.h>
+#include <goffice/gtk/goffice-gtk.h>
 
 #include <gsf/gsf-impl-utils.h>
-#include <gtk/gtknotebook.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkhbox.h>
+#include <glib/gi18n.h>
+
 #include <gtk/gtkalignment.h>
+#include <gtk/gtkhbox.h>
+#include <gtk/gtklabel.h>
+#include <gtk/gtknotebook.h>
+#include <gtk/gtkspinbutton.h>
 
 struct _GogLabel {
-	GogOutlinedObject	base;
+	GogOutlinedObject base;
 
 	GogDatasetElement text;
 	gboolean	  allow_markup;
@@ -48,7 +51,7 @@ typedef GogStyledObjectClass GogLabelClass;
 
 enum {
 	LABEL_PROP_0,
-	LABEL_PROP_ALLOW_MARKUP,
+	LABEL_PROP_ALLOW_MARKUP
 };
 
 static GType gog_label_view_get_type (void);
@@ -115,6 +118,7 @@ gog_label_populate_editor (GogObject *gobj,
 	gtk_widget_show_all (alignment);
 
 	gog_editor_add_page (editor, alignment, _("Data"));
+	
 	(GOG_OBJECT_CLASS(label_parent_klass)->populate_editor) (gobj, editor, dalloc, cc);
 	gog_editor_set_store_page (editor, &label_pref_page);
 }
@@ -122,9 +126,21 @@ gog_label_populate_editor (GogObject *gobj,
 static void
 gog_label_init_style (GogStyledObject *gso, GogStyle *style)
 {
-	style->interesting_fields = GOG_STYLE_OUTLINE | GOG_STYLE_FILL | GOG_STYLE_FONT;
+	GogObject *parent;
+
+	style->interesting_fields = GOG_STYLE_OUTLINE | GOG_STYLE_FILL | 
+		GOG_STYLE_FONT | GOG_STYLE_TEXT_LAYOUT;
 	gog_theme_fillin_style (gog_object_get_theme (GOG_OBJECT (gso)),
 		style, GOG_OBJECT (gso), 0, FALSE);
+	
+	/* Kludge for default Y axis title orientation. This should have be done
+	 * in GogTheme, but it's not possible without breaking graph persistence
+	 * compatibility */
+	parent = gog_object_get_parent (GOG_OBJECT (gso));
+	if (IS_GOG_AXIS (parent) &&
+	    gog_axis_get_atype (GOG_AXIS (parent)) == GOG_AXIS_Y &&
+	    style->text_layout.auto_angle) 
+		style->text_layout.angle = 90.0;
 }
 
 static void
@@ -138,14 +154,15 @@ gog_label_class_init (GogLabelClass *klass)
 	gobject_klass->set_property = gog_label_set_property;
 	gobject_klass->get_property = gog_label_get_property;
 	gobject_klass->finalize	    = gog_label_finalize;
+
 	g_object_class_install_property (gobject_klass, LABEL_PROP_ALLOW_MARKUP,
-		g_param_spec_boolean ("allow-markup", "allow-markup",
+		g_param_spec_boolean ("allow-markup", "Allow markup",
 			"Support basic html-ish markup",
 			TRUE, G_PARAM_READWRITE|GOG_PARAM_PERSISTENT));
 
 	gog_klass->populate_editor	= gog_label_populate_editor;
-	gog_klass->view_type	= gog_label_view_get_type ();
-	style_klass->init_style = gog_label_init_style;
+	gog_klass->view_type		= gog_label_view_get_type ();
+	style_klass->init_style 	= gog_label_init_style;
 }
 
 static void
@@ -176,8 +193,8 @@ gog_label_dataset_init (GogDatasetClass *iface)
 }
 
 GSF_CLASS_FULL (GogLabel, gog_label,
-		gog_label_class_init, NULL,
-		GOG_OUTLINED_OBJECT_TYPE, 0,
+		NULL, NULL, gog_label_class_init, NULL,
+		NULL, GOG_OUTLINED_OBJECT_TYPE, 0,
 		GSF_INTERFACE (gog_label_dataset_init, GOG_DATASET_TYPE))
 
 /************************************************************************/
@@ -193,14 +210,17 @@ static void
 gog_label_view_size_request (GogView *v, GogViewRequisition *req)
 {
 	GogLabel *l = GOG_LABEL (v->model);
+	GOGeometryAABR aabr;
 
 	req->w = req->h = 0.;
 	if (l->text.data != NULL) {
 		char const *text = go_data_scalar_get_str (GO_DATA_SCALAR (l->text.data));
 		if (text != NULL) {
 			gog_renderer_push_style (v->renderer, l->base.base.style);
-			gog_renderer_measure_text (v->renderer, text, req);
+			gog_renderer_get_text_AABR (v->renderer, text, &aabr);
 			gog_renderer_pop_style (v->renderer);
+			req->w = aabr.w;
+			req->h = aabr.h;
 		}
 	}
 	lview_parent_klass->size_request (v, req);
@@ -220,16 +240,16 @@ gog_label_view_render (GogView *view, GogViewAllocation const *bbox)
 			double outline = gog_renderer_line_size (
 				view->renderer, goo->base.style->outline.width);
 			if (style->fill.type != GOG_FILL_STYLE_NONE || outline > 0.) {
-				GogViewRequisition req;
 				GogViewAllocation rect;
+				GOGeometryAABR aabr;
 				double pad_x = gog_renderer_pt2r_x (view->renderer, goo->padding_pts);
 				double pad_y = gog_renderer_pt2r_y (view->renderer, goo->padding_pts);
 				
-				gog_renderer_measure_text (view->renderer, text, &req);
+				gog_renderer_get_text_AABR (view->renderer, text, &aabr);
 				rect = view->allocation;
-				rect.w = req.w + 2. * outline + pad_x;
-				rect.h = req.h + 2. * outline + pad_y;
-				gog_renderer_draw_sharp_rectangle (view->renderer, &rect, NULL);
+				rect.w = aabr.w + 2. * outline + pad_x;
+				rect.h = aabr.h + 2. * outline + pad_y;
+				gog_renderer_draw_sharp_rectangle (view->renderer, &rect);
 			}
 			gog_renderer_draw_text (view->renderer, text,
 						&view->residual, GTK_ANCHOR_NW, NULL);
