@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 #ifndef GOG_OBJECT_H
@@ -86,7 +86,7 @@ struct _GogObject {
 	GSList		*children;
 
 	GogObjectPosition  position;
-	GogViewAllocation *manual_position;
+	GogViewAllocation  manual_position;
 
 	unsigned needs_update : 1;
 	unsigned being_updated : 1;
@@ -100,6 +100,7 @@ typedef struct {
 	GType	    view_type;
 
 	unsigned use_parent_as_proxy : 1; /* when we change, pretend it was our parent */
+	unsigned can_manual_size : 1; 
 
 	/* Virtuals */
 	void	     (*update)		(GogObject *obj);
@@ -132,7 +133,9 @@ typedef struct {
 
 GType gog_object_get_type (void);
 
-GogObject   *gog_object_dup		 (GogObject const *obj, GogObject *new_parent);
+typedef void (*GogDataDuplicator) (GogDataset const *src, GogDataset *dst);
+
+GogObject   *gog_object_dup		 (GogObject const *obj, GogObject *new_parent, GogDataDuplicator dup);
 GogObject   *gog_object_get_parent	 (GogObject const *obj);
 GogObject   *gog_object_get_parent_typed (GogObject const *obj, GType t);
 GogGraph    *gog_object_get_graph	 (GogObject const *obj);
@@ -155,8 +158,15 @@ void		  gog_object_can_reorder (GogObject const *obj,
 					  gboolean *inc_ok, gboolean *dec_ok);
 GogObject	 *gog_object_reorder	 (GogObject const *obj,
 					  gboolean inc, gboolean goto_max);
-GogObjectPosition gog_object_get_pos	 (GogObject const *obj);
-gboolean	  gog_object_set_pos	 (GogObject *obj, GogObjectPosition p);
+
+GogObjectPosition gog_object_get_position_flags	   (GogObject const *obj, GogObjectPosition mask);
+gboolean	  gog_object_set_position_flags	   (GogObject *obj, GogObjectPosition p, GogObjectPosition mask);
+void	          gog_object_get_manual_position   (GogObject *obj, GogViewAllocation *pos);
+void 		  gog_object_set_manual_position   (GogObject *obj, GogViewAllocation const *pos);
+
+GogViewAllocation gog_object_get_manual_allocation (GogObject *gobj, 
+						    GogViewAllocation const *parent_allocation, 
+						    GogViewRequisition const *requisition);
 
 GogObjectRole const *gog_object_find_role_by_name (GogObject const *obj,
 						   char const *role);
@@ -170,6 +180,7 @@ gboolean gog_object_set_parent	 	(GogObject *child, GogObject *parent,
 					 GogObjectRole const *role, unsigned id);
 void 	 gog_object_register_roles	(GogObjectClass *klass,
 					 GogObjectRole const *roles, unsigned n_roles);
+void 	 gog_object_unregister_role	(GogObjectClass *klass, char const *id);
 
 G_END_DECLS
 
