@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -50,6 +50,8 @@ enum {
 	PLOT_PROP_VARY_STYLE_BY_ELEMENT,
 	PLOT_PROP_AXIS_X,
 	PLOT_PROP_AXIS_Y,
+	PLOT_PROP_GROUP,
+	PLOT_PROP_GURU_HINTS
 };
 
 static GObjectClass *plot_parent_klass;
@@ -65,6 +67,10 @@ gog_plot_finalize (GObject *obj)
 	g_slist_free (plot->series); /* GogObject does the unref */
 
 	gog_plot_axis_clear (plot, GOG_AXIS_SET_ALL); /* just in case */
+	if (plot->plot_group)
+		g_free (plot->plot_group);
+	if (plot->guru_hints != NULL)
+		g_free (plot->guru_hints);
 
 	(*plot_parent_klass->finalize) (obj);
 }
@@ -182,8 +188,10 @@ gog_plot_populate_editor (GogObject *obj,
 
 	g_return_if_fail (chart != NULL);
 
-	if (gog_chart_get_axis_set (chart) != GOG_AXIS_SET_XY)
+	if (gog_chart_get_axis_set (chart) != GOG_AXIS_SET_XY) {
+		(GOG_OBJECT_CLASS(plot_parent_klass)->populate_editor) (obj, editor, dalloc, cc);
 		return;
+	}
 
 	table = gtk_table_new (0, 1, FALSE);
 	for (type = 0 ; type < GOG_AXIS_TYPES ; type++) {
@@ -239,6 +247,8 @@ gog_plot_populate_editor (GogObject *obj,
 	}
 	else
 		g_object_unref (G_OBJECT (table));
+	
+	(GOG_OBJECT_CLASS(plot_parent_klass)->populate_editor) (obj, editor, dalloc, cc);
 }
 
 static void
@@ -263,6 +273,18 @@ gog_plot_set_property (GObject *obj, guint param_id,
 	case PLOT_PROP_AXIS_Y:
 		gog_plot_set_axis_by_id (plot, GOG_AXIS_Y, g_value_get_uint (value));
 		break;
+	case PLOT_PROP_GROUP: {
+		char const *group = g_value_get_string (value);
+		if (plot->plot_group)
+			g_free (plot->plot_group);
+		plot->plot_group = (group)? g_strdup (g_value_get_string (value)): NULL;
+		break;
+	}
+	case PLOT_PROP_GURU_HINTS:
+		if (plot->guru_hints != NULL)
+			g_free (plot->guru_hints);
+		plot->guru_hints = g_strdup (g_value_get_string (value));
+		break;
 
 	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
 		 return; /* NOTE : RETURN */
@@ -286,6 +308,12 @@ gog_plot_get_property (GObject *obj, guint param_id,
 	case PLOT_PROP_AXIS_Y:
 		g_value_set_uint (value, gog_plot_get_axis_id (plot, GOG_AXIS_Y));
 		break;
+	case PLOT_PROP_GROUP:
+		g_value_set_string (value, plot->plot_group);
+		break;
+	case PLOT_PROP_GURU_HINTS:
+		g_value_set_string (value, plot->guru_hints);
+		break;
 
 	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
 		 break;
@@ -318,12 +346,15 @@ gog_plot_class_init (GogObjectClass *gog_klass)
 		  role_series_post_add, role_series_pre_remove, NULL },
 	};
 	GObjectClass *gobject_klass = (GObjectClass *) gog_klass;
+	GogPlotClass *plot_klass = (GogPlotClass *) gog_klass;
 
 	plot_parent_klass = g_type_class_peek_parent (gog_klass);
 	gobject_klass->finalize		= gog_plot_finalize;
 	gobject_klass->set_property	= gog_plot_set_property;
 	gobject_klass->get_property	= gog_plot_get_property;
 	gog_klass->populate_editor	= gog_plot_populate_editor;
+	plot_klass->axis_set 		= GOG_AXIS_SET_NONE;
+	plot_klass->guru_helper		= NULL;
 
 	g_object_class_install_property (gobject_klass, PLOT_PROP_VARY_STYLE_BY_ELEMENT,
 		g_param_spec_boolean ("vary-style-by-element", "vary-style-by-element",
@@ -338,6 +369,15 @@ gog_plot_class_init (GogObjectClass *gog_klass)
 		g_param_spec_uint ("y_axis", "y_axis", "Reference to Y axis",
 			0, G_MAXINT, 0,
 			G_PARAM_READWRITE|GOG_PARAM_PERSISTENT));
+	g_object_class_install_property (gobject_klass, PLOT_PROP_GROUP,
+		g_param_spec_string ("plot-group", _("Plot group"), 
+			_("Name of plot group if any"),
+			NULL, G_PARAM_READWRITE|GOG_PARAM_PERSISTENT));
+	g_object_class_install_property (gobject_klass, PLOT_PROP_GURU_HINTS,
+		g_param_spec_string ("guru-hints", _("Guru hints"), 
+			_("Semicolon separated list of hints for automatic addition of objects in"
+			  "guru dialog"),
+			NULL, G_PARAM_READWRITE));
 
 	gog_klass->children_reordered = gog_plot_children_reordered;
 	gog_object_register_roles (gog_klass, roles, G_N_ELEMENTS (roles));
@@ -352,6 +392,8 @@ gog_plot_init (GogPlot *plot, GogPlotClass const *derived_plot_klass)
 	/* start as true so that we can queue an update when it changes */
 	plot->cardinality_valid = TRUE;
 	plot->render_before_axes = FALSE;
+	plot->plot_group = NULL;
+	plot->guru_hints = NULL;
 }
 
 GSF_CLASS_ABSTRACT (GogPlot, gog_plot,
@@ -583,7 +625,7 @@ gog_plot_get_axis_bounds (GogPlot *plot, GogAxisType axis,
 	bounds->logical.maxima = go_nan;
 	bounds->logical.minima = go_nan;
 	bounds->is_discrete = FALSE;
-	bounds->center_on_ticks = FALSE;
+	bounds->center_on_ticks = TRUE;
 	if (klass->axis_get_bounds == NULL)
 		return NULL;
 	return (klass->axis_get_bounds) (plot, axis, bounds);
@@ -606,21 +648,17 @@ gog_plot_axis_set_pref (GogPlot const *plot)
 {
 	GogPlotClass *klass = GOG_PLOT_GET_CLASS (plot);
 
-	g_return_val_if_fail (klass != NULL, FALSE);
-	if (klass->axis_set_pref != NULL)
-		return (klass->axis_set_pref) (plot);
-	return GOG_AXIS_SET_NONE;
+	g_return_val_if_fail (klass != NULL, GOG_AXIS_SET_UNKNOWN);
+	return klass->axis_set;
 }
 
 gboolean
-gog_plot_axis_set_is_valid (GogPlot const *plot, GogAxisSet type)
+gog_plot_axis_set_is_valid (GogPlot const *plot, GogAxisSet axis_set)
 {
 	GogPlotClass *klass = GOG_PLOT_GET_CLASS (plot);
 
 	g_return_val_if_fail (klass != NULL, FALSE);
-	if (klass->axis_set_is_valid != NULL)
-		return (klass->axis_set_is_valid) (plot, type);
-	return type == GOG_AXIS_SET_NONE;
+	return (axis_set == klass->axis_set);
 }
 
 static gboolean
@@ -683,10 +721,7 @@ gog_plot_axis_set_assign (GogPlot *plot, GogAxisSet axis_set)
 		}
 	}
 
-	if (klass->axis_set_assign == NULL)
-		return axis_set == GOG_AXIS_SET_NONE;
-
-	return (klass->axis_set_assign) (plot, axis_set);
+	return (axis_set == klass->axis_set);
 }
 
 /**
@@ -737,6 +772,58 @@ gog_plot_update_3d (GogPlot *plot)
 		klass->update_3d (plot);
 }
 
+static void
+gog_plot_guru_helper_add_grid_line (GogPlot *plot, gboolean major) 
+{
+	GogAxisType type;
+
+	for (type = 0; type < GOG_AXIS_TYPES; type++) {
+		if (((type & (GOG_AXIS_X | 
+			    GOG_AXIS_Y | 
+			    GOG_AXIS_CIRCULAR | 
+			    GOG_AXIS_RADIAL)) != 0) &&
+		    plot->axis[type] != NULL &&
+		    gog_axis_get_grid_line (plot->axis[type], major) == NULL)
+		{
+			gog_object_add_by_name (GOG_OBJECT (plot->axis[type]), 
+						major ? "MajorGrid": "MinorGrid", NULL);
+		}
+	}
+}
+
+void
+gog_plot_guru_helper (GogPlot *plot)
+{
+	GogPlotClass *klass;
+	char **hints;
+	char *hint;
+	unsigned i;
+
+ 	g_return_if_fail (GOG_PLOT (plot) != NULL);	
+	klass = GOG_PLOT_GET_CLASS (plot);
+
+	if (plot->guru_hints == NULL)
+		return;
+
+	hints = g_strsplit (plot->guru_hints, ";", 0);
+	
+	for (i = 0; i < g_strv_length (hints); i++) {
+		hint = g_strstrip (hints[i]);
+		if (strcmp (hints[i], "backplane") == 0) {
+			GogChart *chart = GOG_CHART (gog_object_get_parent (GOG_OBJECT (plot)));
+			if (chart != NULL && gog_chart_get_grid (chart) == NULL)
+				gog_object_add_by_name (GOG_OBJECT (chart), "Grid", NULL);
+		} else if (strcmp (hints[i], "major-grid") == 0) {
+			gog_plot_guru_helper_add_grid_line (plot, TRUE);
+		} else if (strcmp (hints[i], "minor-grid") == 0) {
+			gog_plot_guru_helper_add_grid_line (plot, FALSE);
+		} else if (klass->guru_helper)
+			klass->guru_helper (plot, hint);
+	}
+
+	g_strfreev (hints);
+}
+
 /****************************************************************************/
 /* a placeholder.  It seems likely that we will want this eventually        */
 GSF_CLASS_ABSTRACT (GogPlotView, gog_plot_view,
