@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -204,6 +204,25 @@ GSF_CLASS (GogSeriesElement, gog_series_element,
 	   gog_series_element_class_init, NULL /*gog_series_element_init*/,
 	   GOG_STYLED_OBJECT_TYPE)
 
+/*****************************************************************************/
+
+static gboolean
+regression_curve_can_add (GogObject const *parent)
+{
+	GogSeries *series = GOG_SERIES (parent);
+	return (series->acceptable_children & GOG_SERIES_ACCEPT_REGRESSION_CURVE) != 0;
+}
+
+static void
+regression_curve_post_add (GogObject *parent, GogObject *child)
+{
+	gog_object_request_update (child);
+}
+
+static void
+regression_curve_pre_remove (GogObject *parent, GogObject *child)
+{
+}
 
 /*****************************************************************************/
 
@@ -441,14 +460,6 @@ gog_series_init_style (GogStyledObject *gso, GogStyle *style)
 static void
 gog_series_class_init (GogSeriesClass *klass)
 {
-	static GogObjectRole const roles[] = {
-		{ N_("Point"), "GogSeriesElement",	0,
-		  GOG_POSITION_SPECIAL, GOG_POSITION_SPECIAL, GOG_OBJECT_NAME_BY_ROLE,
-		  role_series_element_can_add, NULL,
-		  role_series_element_allocate,
-		  role_series_element_post_add,
-		  role_series_element_pre_remove, NULL },
-	};
 	GObjectClass *gobject_klass = (GObjectClass *) klass;
 	GogObjectClass *gog_klass = (GogObjectClass *) klass;
 	GogStyledObjectClass *style_klass = (GogStyledObjectClass *) klass;
@@ -464,8 +475,6 @@ gog_series_class_init (GogSeriesClass *klass)
 	/* series do not have views, so just forward signals from the plot */
 	gog_klass->use_parent_as_proxy  = TRUE;
 
-	gog_object_register_roles (gog_klass, roles, G_N_ELEMENTS (roles));
-
 	g_object_class_install_property (gobject_klass, SERIES_HAS_LEGEND,
 		g_param_spec_boolean ("has-legend", "has-legend",
 			"Should the series show up in legends",
@@ -473,6 +482,30 @@ gog_series_class_init (GogSeriesClass *klass)
 			G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
 }
 
+static void
+gog_series_base_init (GogObjectClass *klass)
+{
+	static GogObjectRole const roles[] = {
+		{ N_("Point"), "GogSeriesElement",	0,
+		  GOG_POSITION_SPECIAL, GOG_POSITION_SPECIAL, GOG_OBJECT_NAME_BY_ROLE,
+		  role_series_element_can_add, NULL,
+		  role_series_element_allocate,
+		  role_series_element_post_add,
+		  role_series_element_pre_remove, NULL },
+		{ N_("Regression curve"), "GogRegCurve",	1,
+		  GOG_POSITION_SPECIAL, GOG_POSITION_SPECIAL, GOG_OBJECT_NAME_BY_TYPE,
+		  regression_curve_can_add,
+		  NULL,
+		  NULL,
+		  regression_curve_post_add,
+		  regression_curve_pre_remove,
+		  NULL },
+	};
+
+	klass->roles = NULL;
+	gog_object_register_roles (klass, roles, G_N_ELEMENTS (roles));
+}
+
 static void
 gog_series_init (GogSeries *series)
 {
@@ -481,6 +514,7 @@ gog_series_init (GogSeries *series)
 	series->plot = NULL;
 	series->values = NULL;
 	series->index = -1;
+	series->acceptable_children = 0;
 }
 
 static void
@@ -582,8 +616,8 @@ gog_series_dataset_init (GogDatasetClass *iface)
 }
 
 GSF_CLASS_FULL (GogSeries, gog_series,
-		gog_series_class_init, gog_series_init,
-		GOG_STYLED_OBJECT_TYPE, 0,
+		gog_series_base_init, NULL, gog_series_class_init, NULL,
+		gog_series_init, GOG_STYLED_OBJECT_TYPE, 0,
 		GSF_INTERFACE (gog_series_dataset_init, GOG_DATASET_TYPE))
 
 /**
