@@ -118,6 +118,11 @@ static gboolean boolean_cached = FALSE;
 static char const *lc_TRUE = NULL;
 static char const *lc_FALSE = NULL;
 
+static double beyond_precision;
+#ifdef GOFFICE_WITH_LONG_DOUBLE
+static long double beyond_precisionl;
+#endif
+
 /* FIXME : use nl_langinfo
  * DAY_{1-7} (LC_TIME)
  * ABDAY_{1-7} (LC_TIME)
@@ -356,6 +361,12 @@ format_get_col_sep (void)
 	return ',';
 }
 
+char
+format_get_row_sep (void)
+{
+	return ';';
+}
+
 char const *
 format_boolean (gboolean b)
 {
@@ -857,11 +868,6 @@ lookup_color (gchar const *str, gchar const *end)
 }
 #endif
 
-static double beyond_precision;
-#ifdef GOFFICE_WITH_LONG_DOUBLE
-static long double beyond_precisionl;
-#endif
-
 void
 SUFFIX(go_render_number) (GString *result,
 			  DOUBLE number,
@@ -1237,12 +1243,12 @@ style_format_account (FormatCharacteristics const *fmt)
 		g_string_append_printf (str, "_(%s%s_);_(%s(%s);_(%s\"-\"%s_);_(@_)",
 					sym->str, num->str,
 					sym->str, num->str,
-					sym->str, qmarks + NUM_ZEROS-fmt->num_decimals);
+					sym->str, (qmarks + NUM_ZEROS) - fmt->num_decimals);
 	} else {
 		g_string_append_printf (str, "_(%s%s_);_((%s)%s;_(\"-\"%s%s_);_(@_)",
 					num->str, sym->str,
 					num->str, sym->str,
-					qmarks + NUM_ZEROS-fmt->num_decimals, sym->str);
+					(qmarks + NUM_ZEROS) - fmt->num_decimals, sym->str);
 	}
 
 	g_string_free (num, TRUE);
@@ -1341,11 +1347,9 @@ reformat_decimals (const FormatCharacteristics *fc,
 GOFormat *
 format_remove_decimal (GOFormat const *fmt)
 {
-	int offset = 1;
-	char *ret, *p;
-	char const *tmp;
+	int start;
+	char *ret;
 	char const *format_string = fmt->format;
-	GOFormat *sf;
 
 	switch (fmt->family) {
 	case FMT_NUMBER:
@@ -1392,7 +1396,7 @@ format_remove_decimal (GOFormat const *fmt)
 		; /* Nothing.  */
 	}
 
-	/* Use the old code for more special formats to try to add a
+	/* Use the old code for more special formats to try to remove a
 	   decimal */
 
 	/*
@@ -1404,27 +1408,37 @@ format_remove_decimal (GOFormat const *fmt)
 	if (style_format_is_general (fmt))
 		format_string = "0.########";
 
-	tmp = find_decimal_char (format_string);
-	if (!tmp)
-		return NULL;
-
+	start = 0;
 	ret = g_strdup (format_string);
-	p = ret + (tmp - format_string);
+	while (1) {
+		char *p = (char *)find_decimal_char (ret + start);
+		int offset;
 
-	/* If there is more than 1 thing after the decimal place
-	 * leave the decimal.
-	 * If there is only 1 thing after the decimal remove the decimal too.
-	 */
-	if ((p[1] == '0' || p[1] == '#') && (p[2] == '0' || p[2] == '#'))
-		++p;
-	else
-		offset = 2;
+		if (!p)
+			break;
 
-	strcpy (p, p + offset);
+		/* If there is more than 1 thing after the decimal place
+		 * leave the decimal.
+		 * If there is only 1 thing after the decimal remove the decimal too.
+		 */
+		if ((p[1] == '0' || p[1] == '#') && (p[2] == '0' || p[2] == '#'))
+			offset = 1, ++p;
+		else
+			offset = 2;
 
-	sf = style_format_new_XL (ret, FALSE);
-	g_free (ret);
-	return sf;
+		strcpy (p, p + offset);
+
+		start = (p + 1) - ret;
+	}
+
+	if (start) {
+		GOFormat *sf = style_format_new_XL (ret, FALSE);
+		g_free (ret);
+		return sf;
+	} else {
+		g_free (ret);
+		return NULL;
+	}
 }
 #endif
 
