@@ -17,7 +17,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -43,6 +43,7 @@
 
 #include <string.h>
 #include <stdlib.h>
+#include <unistd.h>
 
 /* ------------------------------------------------------------------------- */
 
@@ -124,7 +125,7 @@ go_shell_argv_to_glib_encoding (gint argc, gchar const **argv)
 		wargs = CommandLineToArgvW (GetCommandLineW (), &narg);
 		conv = g_iconv_open ("utf-8", "utf-16le");
 		for (i = 0; i < narg; ++i)
-			args[i] = g_convert_with_iconv ((const gchar *) wargs[i], wcslen (wargs[i]) << 1, conv, NULL, NULL, NULL);	
+			args[i] = g_convert_with_iconv ((const gchar *) wargs[i], wcslen (wargs[i]) << 1, conv, NULL, NULL, NULL);
 		g_iconv_close (conv);
 	}
 	else
@@ -232,6 +233,28 @@ go_dirname_from_uri (const char *uri, gboolean brief)
 
 /* ------------------------------------------------------------------------- */
 
+static gboolean
+is_fd_uri (const char *uri, int *fd)
+{
+	unsigned long ul;
+	char *end;
+
+	if (g_ascii_strncasecmp (uri, "fd://", 5))
+		return FALSE;
+	uri += 5;
+	if (!g_ascii_isdigit (*uri))
+		return FALSE;  /* Space, for example.  */
+
+	ul = strtoul (uri, &end, 10);
+	if (*end != 0 || ul > INT_MAX)
+		return FALSE;
+
+	*fd = (int)ul;
+	return TRUE;
+}
+
+/* ------------------------------------------------------------------------- */
+
 static GsfInput *
 open_plain_file (const char *path, GError **err)
 {
@@ -254,6 +277,7 @@ GsfInput *
 go_file_open (char const *uri, GError **err)
 {
 	char *filename;
+	int fd;
 
 	if (err != NULL)
 		*err = NULL;
@@ -271,12 +295,23 @@ go_file_open (char const *uri, GError **err)
 		return result;
 	}
 
+	if (is_fd_uri (uri, &fd)) {
+		int fd2 = dup (fd);
+		FILE *fil = fd2 != -1 ? fdopen (fd2, "rb") : NULL;
+		GsfInput *result = fil ? gsf_input_stdio_new_FILE (uri, fil, FALSE) : NULL;
+
+		if (!result)
+			g_set_error (err, gsf_output_error_id (), 0,
+				     "Unable to read from %s", uri);
+		return result;
+	}
+
 #ifdef WITH_GNOME
 	return gsf_input_gnomevfs_new (uri, err);
 #else
 	g_set_error (err, gsf_input_error (), 0,
 		     "Invalid or non-supported URI");
-	return NULL; 
+	return NULL;
 #endif
 }
 
@@ -284,6 +319,7 @@ GsfOutput *
 go_file_create (char const *uri, GError **err)
 {
 	char *filename;
+	int fd;
 
 	g_return_val_if_fail (uri != NULL, NULL);
 
@@ -294,12 +330,23 @@ go_file_create (char const *uri, GError **err)
 		return result;
 	}
 
+	if (is_fd_uri (uri, &fd)) {
+		int fd2 = dup (fd);
+		FILE *fil = fd2 != -1 ? fdopen (fd2, "wb") : NULL;
+		GsfOutput *result = fil ? gsf_output_stdio_new_FILE (uri, fil, FALSE) : NULL;
+
+		if (!result)
+			g_set_error (err, gsf_output_error_id (), 0,
+				     "Unable to write to %s", uri);
+		return result;
+	}
+
 #ifdef WITH_GNOME
 	return gsf_output_gnomevfs_new (uri, err);
 #else
 	g_set_error (err, gsf_output_error_id (), 0,
 		     "Invalid or non-supported URI");
-	return NULL; 
+	return NULL;
 #endif
 }
 
@@ -438,7 +485,7 @@ GError *
 go_url_show (gchar const *url)
 {
 #ifdef G_OS_WIN32
-	ShellExecute (NULL, "open", url, NULL, NULL, SW_SHOWNORMAL);  
+	ShellExecute (NULL, "open", url, NULL, NULL, SW_SHOWNORMAL);
 
 	return NULL;
 #else
@@ -524,7 +571,7 @@ go_url_show (gchar const *url)
  * If no @std_ext is given or @uri already has some extension,
  * it just copies @uri.
  *
- * Value in new_uri:  newly allocated string which you should free after 
+ * Value in new_uri:  newly allocated string which you should free after
  *                    use, containing (optionally) modified uri.
  *
  * Return Value:  FALSE if the uri has an extension not matching @std_ext
@@ -571,7 +618,9 @@ go_get_mime_type (gchar const *uri)
 		g_free (wuri);
 		return g_utf16_to_utf8 (mime_type, -1, NULL, NULL, NULL);
 	}
-	
+
+	g_free (wuri);
+
 	/* We try to determine mime using FindMimeFromData().
 	 * However, we are not sure whether the functions will know about
 	 * the necessary mime types. In the worst wase we fall back to
@@ -579,7 +628,7 @@ go_get_mime_type (gchar const *uri)
 	 */
 	return g_strdup ("text/plain");
 #else
-	return "application/octet-string";
+	return g_strdup ("application/octet-stream");
 #endif
 }
 
