@@ -15,12 +15,13 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
 #include <goffice/goffice-config.h>
 #include "gog-barcol.h"
+#include <goffice/graph/gog-series-lines.h>
 #include <goffice/graph/gog-view.h>
 #include <goffice/graph/gog-renderer.h>
 #include <goffice/graph/gog-style.h>
@@ -107,6 +108,7 @@ gog_barcol_plot_populate_editor (GogObject *item,
 	gog_editor_add_page (editor,
 			     gog_barcol_plot_pref (GOG_BARCOL_PLOT (item), cc),
 			     _("Properties"));
+	(GOG_OBJECT_CLASS(gog_barcol_parent_klass)->populate_editor) (item, editor, dalloc, cc);
 }
 
 static gboolean
@@ -178,6 +180,7 @@ gog_barcol_axis_get_bounds (GogPlot *plot, GogAxisType axis,
 		bounds->val.minima -= .5;
 		bounds->val.maxima += .5;
 		bounds->logical.minima = -.5;
+		bounds->center_on_ticks = FALSE;
 	}
 
 	return data;
@@ -224,11 +227,12 @@ static void
 gog_barcol_plot_init (GogBarColPlot *model)
 {
 	model->gap_percentage = 150;
+	GOG_PLOT1_5D (model)->support_series_lines = TRUE;
 }
 
-GSF_CLASS (GogBarColPlot, gog_barcol_plot,
-	   gog_barcol_plot_class_init, gog_barcol_plot_init,
-	   GOG_PLOT1_5D_TYPE)
+GSF_DYNAMIC_CLASS (GogBarColPlot, gog_barcol_plot,
+	gog_barcol_plot_class_init, gog_barcol_plot_init,
+	GOG_PLOT1_5D_TYPE)
 
 /*****************************************************************************/
 typedef GogPlotView		GogBarColView;
@@ -281,8 +285,7 @@ barcol_draw_rect (GogRenderer *rend, gboolean flip,
 	path[5].code = ART_END;
 	
 	gog_renderer_draw_sharp_polygon (rend, path, 
-					 fabs (x1 - x0) < 3. || fabs (y1 - y0) < 3.
-					 , NULL);
+					 fabs (x1 - x0) < 3. || fabs (y1 - y0) < 3.);
 }
 
 typedef struct {
@@ -312,9 +315,13 @@ gog_barcol_view_render (GogView *view, GogViewAllocation const *bbox)
 	GogStyle **styles;
 	ErrorBarData **error_data;
 	GogErrorBar **errors;
+	GogSeriesLines **lines;
+	ArtVpath **paths;
 	GSList *ptr;
 	unsigned *lengths;
 	double plus, minus;
+	GogObjectRole const *role = NULL;
+
 
 	if (num_elements <= 0 || num_series <= 0)
 		return;
@@ -337,6 +344,8 @@ gog_barcol_view_render (GogView *view, GogViewAllocation const *bbox)
 	styles = g_alloca (num_series * sizeof (GogStyle *));
 	errors = g_alloca (num_series * sizeof (GogErrorBar *));
 	error_data = g_alloca (num_series * sizeof (ErrorBarData *));
+	lines = g_alloca (num_series * sizeof (GogSeriesLines *));
+	paths = g_alloca (num_series * sizeof (ArtVpath *));
 	
 	i = 0;
 	for (ptr = gog_1_5d_model->base.series ; ptr != NULL ; ptr = ptr->next) {
@@ -353,6 +362,20 @@ gog_barcol_view_render (GogView *view, GogViewAllocation const *bbox)
 			error_data[i] = g_malloc (sizeof (ErrorBarData) * lengths[i]);
 		else 
 			error_data[i] = NULL;
+		if (series->has_series_lines) {
+			if (!role)
+				role = gog_object_find_role_by_name (
+							GOG_OBJECT (series), "Series lines");
+			lines[i] = GOG_SERIES_LINES (
+					gog_object_get_child_by_role (GOG_OBJECT (series), role));
+			paths[i] = g_new (ArtVpath, lengths[i] * 2 - 1);
+			for (j = 0; j < lengths[i] - 1; j++) {
+				paths[i][j * 2].code = ART_MOVETO;
+				paths[i][j * 2 + 1].code = ART_LINETO;
+			}
+			paths[i][j * 2].code = ART_END;
+		} else
+			lines[i] = NULL;
 		i++;
 	}
 
@@ -415,7 +438,7 @@ gog_barcol_view_render (GogView *view, GogViewAllocation const *bbox)
 			if (gog_error_bar_is_visible (errors[j])) {
 				x = tmp > 0 ? work.x + work.w: work.x;
 				error_data[j][i].plus = plus * data_scale;
-				error_data[j][i].minus =minus * data_scale;
+				error_data[j][i].minus = minus * data_scale;
 				if (is_vertical) {
 					error_data[j][i].x = work.y + work.h / 2.0;
 					error_data[j][i].y = x;
@@ -424,6 +447,28 @@ gog_barcol_view_render (GogView *view, GogViewAllocation const *bbox)
 					error_data[j][i].y = work.y + work.h / 2.0;
 				}
 			}
+			if (lines[j] != NULL) {
+				x = tmp > 0 ? work.x + work.w: work.x;
+				if (is_vertical) {
+					if (i > 0) {
+						paths[j][i * 2 - 1].x = gog_axis_map_to_view (x_map,
+												work.y);
+						paths[j][i * 2 - 1].y = gog_axis_map_to_view (y_map, x);
+					}
+					paths[j][i * 2].x = gog_axis_map_to_view (x_map,
+											work.y + work.h);
+					paths[j][i * 2].y = gog_axis_map_to_view (y_map, x);
+				} else {
+					if (i > 0) {
+						paths[j][i * 2 - 1].x = gog_axis_map_to_view (x_map, x);
+						paths[j][i * 2 - 1].y = gog_axis_map_to_view (y_map,
+													work.y);
+					}
+					paths[j][i].x = gog_axis_map_to_view (x_map, x);
+					paths[j][i].x = gog_axis_map_to_view (y_map,
+											work.y + work.h);
+				}
+			}
 		}
 	}
 	/*Now draw error bars and clean*/
@@ -438,6 +483,13 @@ gog_barcol_view_render (GogView *view, GogViewAllocation const *bbox)
 			g_free (error_data[i]);
 		}
 
+	/* Draw series lines if any */
+	for (i = 0; i < num_series; i++)
+		if (lines[i] != NULL) {
+			gog_series_lines_render (lines[i], view->renderer, bbox, paths[i], FALSE);
+			g_free (paths[i]);
+		}
+
 	gog_axis_map_free (x_map);
 	gog_axis_map_free (y_map);
 }
@@ -462,6 +514,6 @@ gog_barcol_view_class_init (GogViewClass *view_klass)
 	view_klass->clip	  = TRUE;
 }
 
-static GSF_CLASS (GogBarColView, gog_barcol_view,
-		  gog_barcol_view_class_init, NULL,
-		  GOG_PLOT_VIEW_TYPE)
+GSF_DYNAMIC_CLASS (GogBarColView, gog_barcol_view,
+	gog_barcol_view_class_init, NULL,
+	GOG_PLOT_VIEW_TYPE)
