@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -197,24 +197,6 @@ gog_box_plot_axis_get_bounds (GogPlot *plot, GogAxisType axis,
 	return NULL;
 }
 
-static GogAxisSet
-gog_box_plot_axis_set_pref (GogPlot const *plot)
-{
-	return GOG_AXIS_SET_X;
-}
-
-static gboolean
-gog_box_plot_axis_set_is_valid (GogPlot const *plot, GogAxisSet type)
-{
-	return type == GOG_AXIS_SET_X;
-}
-
-static gboolean
-gog_box_plot_axis_set_assign (GogPlot *plot, GogAxisSet type)
-{
-	return type == GOG_AXIS_SET_X;
-}
-
 static void
 gog_box_plot_class_init (GogPlotClass *gog_plot_klass)
 {
@@ -247,9 +229,7 @@ gog_box_plot_class_init (GogPlotClass *gog_plot_klass)
 	plot_klass->desc.num_series_min = 1;
 	plot_klass->desc.num_series_max = G_MAXINT;
 	plot_klass->series_type = gog_box_plot_series_get_type ();
-	plot_klass->axis_set_pref     = gog_box_plot_axis_set_pref;
-	plot_klass->axis_set_is_valid = gog_box_plot_axis_set_is_valid;
-	plot_klass->axis_set_assign   = gog_box_plot_axis_set_assign;
+	plot_klass->axis_set = GOG_AXIS_SET_X;
 	plot_klass->desc.series.style_fields	= GOG_STYLE_LINE | GOG_STYLE_FILL;
 	plot_klass->axis_get_bounds   		= gog_box_plot_axis_get_bounds;
 }
@@ -321,30 +301,30 @@ gog_box_plot_view_render (GogView *view, GogViewAllocation const *bbox)
 			rect.w = qu3 - qu1;
 			rect.y = y - hrect;
 			rect.h = 2* hrect;
-			gog_renderer_draw_sharp_rectangle (view->renderer, &rect, NULL);
+			gog_renderer_draw_sharp_rectangle (view->renderer, &rect);
 			path[2].code = ART_END;
 			path[0].y = y + hbar;
 			path[1].y = y - hbar;
 			path[0].x = path[1].x = min;
-			gog_renderer_draw_sharp_path (view->renderer, path, NULL);
+			gog_renderer_draw_sharp_path (view->renderer, path);
 			path[0].x = path[1].x = max;
-			gog_renderer_draw_sharp_path (view->renderer, path, NULL);
+			gog_renderer_draw_sharp_path (view->renderer, path);
 			path[0].y = path[1].y = y;
 			path[0].x = qu3;
-			gog_renderer_draw_sharp_path (view->renderer, path, NULL);
+			gog_renderer_draw_sharp_path (view->renderer, path);
 			path[0].x = min;
 			path[1].x = qu1;
-			gog_renderer_draw_sharp_path (view->renderer, path, NULL);
+			gog_renderer_draw_sharp_path (view->renderer, path);
 			path[0].x = path[1].x = med;
 			path[0].y = y + hrect;
 			path[1].y = y - hrect;
-			gog_renderer_draw_sharp_path (view->renderer, path, NULL);
+			gog_renderer_draw_sharp_path (view->renderer, path);
 			path[2].code = ART_LINETO;
 			path[0].x = path[3].x = path[4].x = qu1;
 			path[1].x = path[2].x = qu3;
 			path[0].y = path[1].y = path[4].y = y - hrect;
 			path[2].y = path[3].y = y + hrect;
-			gog_renderer_draw_sharp_path (view->renderer, path, NULL);
+			gog_renderer_draw_sharp_path (view->renderer, path);
 		} else {
 			GogViewAllocation pos;
 			pos.x = view->allocation.x + view->allocation.w / 2;
