@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -23,6 +23,7 @@
 #include "gog-radar.h"
 #include <goffice/graph/gog-axis.h>
 #include <goffice/graph/gog-chart.h>
+#include <goffice/graph/gog-grid-line.h>
 #include <goffice/graph/gog-view.h>
 #include <goffice/graph/gog-renderer.h>
 #include <goffice/graph/gog-theme.h>
@@ -36,6 +37,8 @@
 #include <glib/gi18n.h>
 #include <gsf/gsf-impl-utils.h>
 
+#include <string.h>
+
 typedef struct {
 	GogPlotClass	base;
 } GogRTPlotClass;
@@ -105,24 +108,6 @@ gog_rt_plot_get_property (GObject *obj, guint param_id,
 	}
 }
 
-static GogAxisSet
-gog_rt_plot_axis_set_pref (GogPlot const *plot)
-{
-	return GOG_AXIS_SET_RADAR;
-}
-
-static gboolean
-gog_rt_plot_axis_set_is_valid (GogPlot const *plot, GogAxisSet type)
-{
-	return type == GOG_AXIS_SET_RADAR;
-}
-
-static gboolean
-gog_rt_plot_axis_set_assign (GogPlot *plot, GogAxisSet type)
-{
-	return type == GOG_AXIS_SET_RADAR;
-}
-
 static void
 gog_rt_plot_update (GogObject *obj)
 {
@@ -160,6 +145,21 @@ gog_rt_plot_update (GogObject *obj)
 	gog_object_emit_changed (GOG_OBJECT (obj), FALSE);
 }
 
+static void
+gog_rt_plot_guru_helper (GogPlot *plot, char const *hint)
+{
+	if (strcmp (hint, "circular-no-line") == 0) {
+		GogAxis *axis = gog_plot_get_axis (plot, GOG_AXIS_CIRCULAR);
+		GogStyle *style;
+
+		g_return_if_fail (GOG_AXIS (axis) != NULL);
+
+		style = gog_styled_object_get_style (GOG_STYLED_OBJECT (axis));
+		style->line.dash_type = GO_LINE_NONE;
+		style->line.auto_dash = FALSE;
+	};
+}
+
 static void
 gog_rt_plot_class_init (GogPlotClass *gog_plot_klass)
 {
@@ -185,9 +185,8 @@ gog_rt_plot_class_init (GogPlotClass *gog_plot_klass)
 	gog_plot_klass->desc.num_series_max = G_MAXINT;
 	gog_plot_klass->series_type = gog_rt_series_get_type();
 
-	gog_plot_klass->axis_set_pref = gog_rt_plot_axis_set_pref;
-	gog_plot_klass->axis_set_is_valid = gog_rt_plot_axis_set_is_valid;
-	gog_plot_klass->axis_set_assign = gog_rt_plot_axis_set_assign;
+	gog_plot_klass->axis_set    = GOG_AXIS_SET_RADAR;
+	gog_plot_klass->guru_helper = gog_rt_plot_guru_helper;
 }
 
 static void
@@ -229,6 +228,7 @@ gog_radar_plot_axis_get_bounds (GogPlot *plot, GogAxisType axis,
 		bounds->logical.minima = 0.;
 		bounds->logical.maxima = go_nan;
 		bounds->is_discrete    = TRUE;
+		bounds->center_on_ticks = TRUE;
 
 		for (ptr = plot->series; ptr != NULL ; ptr = ptr->next)
 			if (gog_series_is_valid (GOG_SERIES (ptr->data)))
@@ -405,13 +405,15 @@ gog_rt_view_render (GogView *view, GogViewAllocation const *bbox)
 	GSList   *ptr;
 	ArtVpath *path, *clip_path;
 	ArtBpath *bpath;
+	double theta_min, theta_max, theta;
+	double rho_min, rho_max, rho;
 	gboolean const is_area = GOG_IS_PLOT_RADAR_AREA (model);
 	gboolean const is_polar = GOG_IS_PLOT_POLAR (model);
 
 	r_axis = GOG_PLOT (model)->axis[GOG_AXIS_RADIAL];
 	c_axis = GOG_PLOT (model)->axis[GOG_AXIS_CIRCULAR];
 	g_return_if_fail (r_axis != NULL && c_axis != NULL);
-	
+
 	area = gog_chart_view_get_plot_area (view->parent);
 	chart_map = gog_chart_map_new (chart, area, c_axis, r_axis, NULL, FALSE);
 	if (!gog_chart_map_is_valid (chart_map)) {
@@ -421,12 +423,19 @@ gog_rt_view_render (GogView *view, GogViewAllocation const *bbox)
 	c_map = gog_chart_map_get_axis_map (chart_map, 0);
 	r_map = gog_chart_map_get_axis_map (chart_map, 1);
 	parms = gog_chart_map_get_polar_parms (chart_map);
-
-	bpath = gog_renderer_get_ring_wedge_bpath (parms->cx, parms->cy, parms->rx, parms->ry,
-						   0.0, 0.0, -parms->th0, -parms->th1);
-	clip_path = art_bez_path_to_vec (bpath, .1);
-	g_free (bpath);
 	
+	gog_axis_map_get_bounds (c_map, &theta_max, &theta_min);
+	gog_axis_map_get_bounds (r_map, &rho_min, &rho_max);
+	/* convert theta value to radians */
+	theta_min = gog_axis_map_to_view (c_map, theta_min);
+	theta_max = gog_axis_map_to_view (c_map, theta_max);
+	if (theta_min > theta_max) {
+		/* angles may be inverted */
+		theta = theta_min;
+		theta_min = theta_max;
+		theta_max = theta;
+	}
+
 	path = g_alloca ((model->num_elements + 2) * sizeof (ArtVpath));
 	for (ptr = model->base.series; ptr != NULL; ptr = ptr->next) {
 
@@ -453,15 +462,24 @@ gog_rt_view_render (GogView *view, GogViewAllocation const *bbox)
 				closed_shape = FALSE;
 				continue;
 			}
-
+			
 			path[count].code = ((count != 0 && !isnan (r_vals[count-1])) 
 					    ? ART_LINETO : ART_MOVETO);
+
+			rho = (!is_polar || (go_add_epsilon (r_vals[count] - rho_min) >= 0.0)) ?
+				r_vals[count] : rho_min;
 			gog_chart_map_2D_to_view (chart_map, 
-						  is_polar ? c_vals[count] : count,
-						  r_vals[count], 
+						  is_polar ? c_vals[count] : count, rho,
 						  &path[count].x, &path[count].y);
 
-			gog_renderer_draw_marker (view->renderer, path[count].x, path[count].y);
+			if (is_polar) theta = gog_axis_map_to_view (c_map, c_vals[count]);
+
+			if ( !is_polar || 
+			     (go_add_epsilon (r_vals[count] - rho_min) >= 0.0 && 
+			      go_add_epsilon (rho_max - r_vals[count]) >= 0.0 &&
+			      go_add_epsilon ((theta_max - theta_min) - fmod (theta_max - theta, 2 * M_PI)) >= 0.0 &&
+			      go_add_epsilon ((theta_max - theta_min) - fmod (theta - theta_min, 2 * M_PI)) >= 0.0))
+				gog_renderer_draw_marker (view->renderer, path[count].x, path[count].y);
 		}
 
 		if (!is_polar && series->base.num_elements == model->num_elements
@@ -473,14 +491,24 @@ gog_rt_view_render (GogView *view, GogViewAllocation const *bbox)
 		}
 		path[count].code = ART_END;
 
-		if (closed_shape && is_area)
-			gog_renderer_draw_polygon (view->renderer, path, FALSE, bbox);
-		else
-			gog_renderer_draw_path (view->renderer, path, bbox);
+		if (is_polar) { 
+			bpath = gog_renderer_get_ring_wedge_bpath (parms->cx, parms->cy, parms->rx, parms->ry,
+								   0.0, 0.0, -parms->th0, -parms->th1);
+			clip_path = art_bez_path_to_vec (bpath, .1);
+			g_free (bpath);
+			gog_renderer_push_clip (view->renderer, clip_path);	
+		}
+
+		if (closed_shape && is_area) {
+			gog_renderer_draw_polygon (view->renderer, path, FALSE);
+		} else
+			gog_renderer_draw_path (view->renderer, path);
+
+		if (is_polar)
+			gog_renderer_pop_clip (view->renderer);
 
 		gog_renderer_pop_style (view->renderer);
 	}
-	g_free (clip_path);
 	gog_chart_map_free (chart_map);
 }
 
@@ -553,14 +581,10 @@ gog_rt_series_init_style (GogStyledObject *gso, GogStyle *style)
 		return;
 
 	plot = GOG_RT_PLOT (series->plot);
-	if (!plot->default_style_has_markers) {
-		style->disable_theming |= GOG_STYLE_MARKER;
-		if (style->marker.auto_shape) {
-			GOMarker *m = go_marker_new ();
-			go_marker_set_shape (m, GO_MARKER_NONE);
-			gog_style_set_marker (style, m);
-		}
-	}
+
+	if (!plot->default_style_has_markers &&
+	    style->marker.auto_shape) 
+		go_marker_set_shape (style->marker.mark, GO_MARKER_NONE);
 }
 
 static void
