@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -27,6 +27,8 @@
 #include <goffice/graph/gog-theme.h>
 #include <goffice/graph/gog-axis.h>
 #include <goffice/graph/gog-error-bar.h>
+#include <goffice/graph/gog-chart.h>
+#include <goffice/graph/gog-series-lines.h>
 #include <goffice/data/go-data.h>
 #include <goffice/utils/go-color.h>
 #include <goffice/utils/go-marker.h>
@@ -163,24 +165,6 @@ gog_2d_plot_adjust_bounds (Gog2DPlot *model, double *x_min, double *x_max, doubl
 	klass->adjust_bounds (model, x_min, x_max, y_min, y_max);
 }
 
-static GogAxisSet
-gog_2d_plot_axis_set_pref (GogPlot const *plot)
-{
-	return GOG_AXIS_SET_XY;
-}
-
-static gboolean
-gog_2d_plot_axis_set_is_valid (GogPlot const *plot, GogAxisSet type)
-{
-	return type == GOG_AXIS_SET_XY;
-}
-
-static gboolean
-gog_2d_plot_axis_set_assign (GogPlot *plot, GogAxisSet type)
-{
-	return type == GOG_AXIS_SET_XY;
-}
-
 static GOData *
 gog_2d_plot_axis_get_bounds (GogPlot *plot, GogAxisType axis,
 			     GogPlotBoundInfo *bounds)
@@ -241,11 +225,9 @@ gog_2d_plot_class_init (GogPlotClass *plot_klass)
 
 	plot_klass->desc.num_series_min = 1;
 	plot_klass->desc.num_series_max = G_MAXINT;
-	plot_klass->series_type  = gog_xy_series_get_type ();
-	plot_klass->axis_set_pref     = gog_2d_plot_axis_set_pref;
-	plot_klass->axis_set_is_valid = gog_2d_plot_axis_set_is_valid;
-	plot_klass->axis_set_assign   = gog_2d_plot_axis_set_assign;
-	plot_klass->axis_get_bounds   = gog_2d_plot_axis_get_bounds;
+	plot_klass->series_type  	= gog_xy_series_get_type ();
+	plot_klass->axis_set	      	= GOG_AXIS_SET_XY;
+	plot_klass->axis_get_bounds   	= gog_2d_plot_axis_get_bounds;
 }
 
 static void
@@ -255,9 +237,9 @@ gog_2d_plot_init (Gog2DPlot *plot2d)
 	plot2d->x.fmt = plot2d->y.fmt = NULL;
 }
 
-GSF_CLASS (Gog2DPlot, gog_2d_plot,
-	   gog_2d_plot_class_init, gog_2d_plot_init,
-	   GOG_PLOT_TYPE)
+GSF_DYNAMIC_CLASS (Gog2DPlot, gog_2d_plot,
+	gog_2d_plot_class_init, gog_2d_plot_init,
+	GOG_PLOT_TYPE)
 
 enum {
 	GOG_XY_PROP_0,
@@ -373,9 +355,9 @@ gog_xy_plot_init (GogXYPlot *xy)
 	xy->default_style_has_lines = TRUE;
 }
 
-GSF_CLASS (GogXYPlot, gog_xy_plot,
-	   gog_xy_plot_class_init, gog_xy_plot_init,
-	   GOG_2D_PLOT_TYPE)
+GSF_DYNAMIC_CLASS (GogXYPlot, gog_xy_plot,
+	gog_xy_plot_class_init, gog_xy_plot_init,
+	GOG_2D_PLOT_TYPE)
 
 /*****************************************************************************/
 
@@ -551,9 +533,9 @@ gog_bubble_plot_init (GogBubblePlot *bubble)
 	bubble->bubble_scale = 1.0;
 }
 
-GSF_CLASS (GogBubblePlot, gog_bubble_plot,
-	   gog_bubble_plot_class_init, gog_bubble_plot_init,
-	   GOG_2D_PLOT_TYPE)
+GSF_DYNAMIC_CLASS (GogBubblePlot, gog_bubble_plot,
+	gog_bubble_plot_class_init, gog_bubble_plot_init,
+	GOG_2D_PLOT_TYPE)
 
 /*****************************************************************************/
 typedef GogPlotView		GogXYView;
@@ -580,7 +562,7 @@ bubble_draw_circle (GogView *view, double x, double y, double radius)
 	}
 	path[MAX_ARC_SEGMENTS].code = ART_LINETO;
 	path[MAX_ARC_SEGMENTS + 1].code = ART_END;
-	gog_renderer_draw_polygon (view->renderer, path, FALSE, &view->residual);
+	gog_renderer_draw_polygon (view->renderer, path, FALSE);
 }
 
 typedef struct {
@@ -600,7 +582,7 @@ gog_xy_view_render (GogView *view, GogViewAllocation const *bbox)
 	GSList *ptr;
 	double const *y_vals, *x_vals = NULL, *z_vals = NULL;
 	double x = 0., y = 0., z, x_canvas = 0., y_canvas = 0.;
-	double zmax, rmax = 0.;
+	double zmax, rmax = 0., x_zero, y_zero;
 	double x_margin_min, x_margin_max, y_margin_min, y_margin_max, margin;
 	double xerrmin, xerrmax, yerrmin, yerrmax;
 	GogStyle *style = NULL;
@@ -622,7 +604,17 @@ gog_xy_view_render (GogView *view, GogViewAllocation const *bbox)
 		return;
 	}
 
-	gog_renderer_clip_push (view->renderer, &view->allocation);
+	/* Draw drop lines from point to axis start. To change this behaviour
+	 * and draw drop lines from point to zero, use gog_axis_map_get_baseline:
+	 * x_zero = gog_axis_map_get_baseline (x_map); */
+
+	gog_axis_map_get_extents (x_map, &x_zero, NULL); 
+	x_zero = gog_axis_map_to_view (x_map, x_zero);
+	gog_axis_map_get_extents (y_map, &y_zero, NULL); 
+	y_zero = gog_axis_map_to_view (y_map, y_zero);
+
+	gog_renderer_push_clip (view->renderer, 
+				gog_renderer_get_rectangle_vpath (&view->allocation));
 
 	for (num_series = 0, ptr = model->base.series ; ptr != NULL ; ptr = ptr->next, num_series++);
 	markers = g_alloca (num_series * sizeof (MarkerData *));
@@ -652,6 +644,50 @@ gog_xy_view_render (GogView *view, GogViewAllocation const *bbox)
 		if (n <= 0)
 			continue;
 
+		/* plot drop lines if any */
+		if (series->hdroplines) {
+			ArtVpath droppath[3];
+			droppath[0].code = ART_MOVETO;
+			droppath[1].code = ART_LINETO;
+			droppath[2].code = ART_END;
+			droppath[0].x = x_zero;
+			gog_renderer_push_style (view->renderer,
+				gog_styled_object_get_style (GOG_STYLED_OBJECT (series->hdroplines)));
+			for (i = 0; i < n; i++) {
+				if (!gog_axis_map_finite (y_map, y_vals[i]))
+					continue;
+				x = x_vals ? x_vals[i] : i + 1;
+				if (!gog_axis_map_finite (x_map, x))
+					continue;
+				droppath[1].x = gog_axis_map_to_view (x_map, x);
+				droppath[0].y = droppath[1].y = gog_axis_map_to_view (y_map, y_vals[i]);
+				gog_series_lines_render (GOG_SERIES_LINES (series->hdroplines),
+								view->renderer, bbox, droppath, FALSE);
+			}
+			gog_renderer_pop_style (view->renderer);
+		}
+		if (series->vdroplines) {
+			ArtVpath droppath[3];
+			droppath[0].code = ART_MOVETO;
+			droppath[1].code = ART_LINETO;
+			droppath[2].code = ART_END;
+			droppath[0].y = y_zero;
+			gog_renderer_push_style (view->renderer,
+				gog_styled_object_get_style (GOG_STYLED_OBJECT (series->vdroplines)));
+			for (i = 0; i < n; i++) {
+				if (!gog_axis_map_finite (y_map, y_vals[i]))
+					continue;
+				x = x_vals ? x_vals[i] : i + 1;
+				if (!gog_axis_map_finite (x_map, x))
+					continue;
+				droppath[1].y = gog_axis_map_to_view (y_map, y_vals[i]);
+				droppath[0].x = droppath[1].x = gog_axis_map_to_view (x_map, x);
+				gog_series_lines_render (GOG_SERIES_LINES (series->vdroplines),
+								view->renderer, bbox, droppath, FALSE);
+			}
+			gog_renderer_pop_style (view->renderer);
+		}
+
 		show_marks = gog_style_is_marker_visible (style);
 		show_lines = gog_style_is_line_visible (style);
 		if (!show_marks && !show_lines)
@@ -699,12 +735,12 @@ gog_xy_view_render (GogView *view, GogViewAllocation const *bbox)
 			if (GOG_XY_PLOT (view->model)->use_splines) {
 				ArtBpath *path;
 				path = go_line_build_bpath (x_splines, y_splines, n);
-				gog_renderer_draw_bezier_path (view->renderer, path, &view->residual);
+				gog_renderer_draw_bezier_path (view->renderer, path);
 				art_free (path);
 			} else {
 				ArtVpath *path;
 				path = go_line_build_vpath (x_splines, y_splines, n);
-				gog_renderer_draw_path (view->renderer, path, &view->residual);
+				gog_renderer_draw_path (view->renderer, path);
 				art_free (path);
 			}
 			g_free (x_splines);
@@ -793,7 +829,7 @@ gog_xy_view_render (GogView *view, GogViewAllocation const *bbox)
 			g_object_unref (neg_style);
 	}
 
-	gog_renderer_clip_pop (view->renderer);
+	gog_renderer_pop_clip (view->renderer);
 
 	if (!GOG_IS_BUBBLE_PLOT (model))
 		for (j = 0, ptr = model->base.series ; ptr != NULL ; ptr = ptr->next, j++) {
@@ -810,6 +846,13 @@ gog_xy_view_render (GogView *view, GogViewAllocation const *bbox)
 				}
 			}
 
+	/* Now render children, may be should come before markers? */
+	gog_renderer_push_clip (view->renderer, 
+				gog_renderer_get_rectangle_vpath (&view->residual));
+	for (ptr = view->children ; ptr != NULL ; ptr = ptr->next)
+		gog_view_render	(ptr->data, bbox);
+	gog_renderer_pop_clip (view->renderer);
+
 	gog_axis_map_free (x_map);
 	gog_axis_map_free (y_map);
 }
@@ -822,20 +865,119 @@ gog_xy_view_info_at_point (GogView *view, double x, double y,
 	return FALSE;
 }
 
+static GogViewClass *xy_view_parent_klass;
+
+static void
+gog_xy_view_size_allocate (GogView *view, GogViewAllocation const *allocation)
+{
+	GSList *ptr;
+	for (ptr = view->children; ptr != NULL; ptr = ptr->next)
+		gog_view_size_allocate (GOG_VIEW (ptr->data), allocation);
+	(xy_view_parent_klass->size_allocate) (view, allocation);
+}
+
 static void
 gog_xy_view_class_init (GogViewClass *view_klass)
 {
+	xy_view_parent_klass = (GogViewClass*) g_type_class_peek_parent (view_klass);
 	view_klass->render	  = gog_xy_view_render;
+	view_klass->size_allocate = gog_xy_view_size_allocate;
 	view_klass->info_at_point = gog_xy_view_info_at_point;
 	view_klass->clip	  = FALSE;
 }
 
-static GSF_CLASS (GogXYView, gog_xy_view,
-		  gog_xy_view_class_init, NULL,
-		  GOG_PLOT_VIEW_TYPE)
+GSF_DYNAMIC_CLASS (GogXYView, gog_xy_view,
+	gog_xy_view_class_init, NULL,
+	GOG_PLOT_VIEW_TYPE)
 
 /*****************************************************************************/
 
+typedef GogView		GogXYSeriesView;
+typedef GogViewClass	GogXYSeriesViewClass;
+
+#define GOG_XY_SERIES_VIEW_TYPE	(gog_xy_series_view_get_type ())
+#define GOG_XY_SERIES_VIEW(o)	(G_TYPE_CHECK_INSTANCE_CAST ((o), GOG_XY_SERIES_VIEW_TYPE, GogXYSeriesView))
+#define IS_GOG_XY_SERIES_VIEW(o)	(G_TYPE_CHECK_INSTANCE_TYPE ((o), GOG_XY_SERIES_VIEW_TYPE))
+
+static void
+gog_xy_series_view_render (GogView *view, GogViewAllocation const *bbox)
+{
+	GSList *ptr;
+	for (ptr = view->children ; ptr != NULL ; ptr = ptr->next)
+		gog_view_render	(ptr->data, bbox);
+}
+
+static void
+gog_xy_series_view_size_allocate (GogView *view, GogViewAllocation const *allocation)
+{
+	GSList *ptr;
+
+	for (ptr = view->children; ptr != NULL; ptr = ptr->next)
+		gog_view_size_allocate (GOG_VIEW (ptr->data), allocation);
+}
+
+static void
+gog_xy_series_view_class_init (GogXYSeriesViewClass *gview_klass)
+{
+	GogViewClass *view_klass = GOG_VIEW_CLASS (gview_klass);
+	view_klass->render = gog_xy_series_view_render;
+	view_klass->size_allocate = gog_xy_series_view_size_allocate;
+}
+
+GSF_DYNAMIC_CLASS (GogXYSeriesView, gog_xy_series_view,
+	gog_xy_series_view_class_init, NULL,
+	GOG_VIEW_TYPE)
+
+/*****************************************************************************/
+
+static gboolean
+horiz_drop_lines_can_add (GogObject const *parent)
+{
+	GogXYSeries *series = GOG_XY_SERIES (parent);
+	return (series->hdroplines == NULL);
+}
+
+static void
+horiz_drop_lines_post_add (GogObject *parent, GogObject *child)
+{
+	GogXYSeries *series = GOG_XY_SERIES (parent);
+	series->hdroplines = child;
+	gog_object_request_update (child);
+}
+
+static void
+horiz_drop_lines_pre_remove (GogObject *parent, GogObject *child)
+{
+	GogXYSeries *series = GOG_XY_SERIES (parent);
+	series->hdroplines = NULL;
+}
+
+/*****************************************************************************/
+
+static gboolean
+vert_drop_lines_can_add (GogObject const *parent)
+{
+	GogXYSeries *series = GOG_XY_SERIES (parent);
+	return (series->vdroplines == NULL);
+}
+
+static void
+vert_drop_lines_post_add (GogObject *parent, GogObject *child)
+{
+	GogXYSeries *series = GOG_XY_SERIES (parent);
+	series->vdroplines = child;
+	gog_object_request_update (child);
+}
+
+static void
+vert_drop_lines_pre_remove (GogObject *parent, GogObject *child)
+{
+	GogXYSeries *series = GOG_XY_SERIES (parent);
+	series->vdroplines = NULL;
+}
+
+/****************************************************************************/
+
 typedef GogSeriesClass GogXYSeriesClass;
 
 enum {
@@ -853,6 +995,7 @@ gog_xy_series_update (GogObject *obj)
 	int x_len = 0, y_len = 0;
 	GogXYSeries *series = GOG_XY_SERIES (obj);
 	unsigned old_num = series->base.num_elements;
+	GSList *ptr;
 
 	if (series->base.values[1].data != NULL) {
 		y_vals = go_data_vector_get_values (GO_DATA_VECTOR (series->base.values[1].data));
@@ -877,6 +1020,11 @@ gog_xy_series_update (GogObject *obj)
 		x_len = y_len;
 	series->base.num_elements = MIN (x_len, y_len);
 
+	/* update children */
+	for (ptr = obj->children; ptr != NULL; ptr = ptr->next)
+		if (!IS_GOG_SERIES_LINES (ptr->data))
+			gog_object_request_update (GOG_OBJECT (ptr->data));
+
 	/* queue plot for redraw */
 	gog_object_request_update (GOG_OBJECT (series->base.plot));
 	if (old_num != series->base.num_elements)
@@ -892,6 +1040,9 @@ gog_xy_series_init (GObject *obj)
 	GogXYSeries *series = GOG_XY_SERIES (obj);
 
 	series->x_errors = series->y_errors = NULL;
+	(GOG_SERIES (series))->acceptable_children =
+				GOG_SERIES_ACCEPT_REGRESSION_CURVE;
+	series->hdroplines = series->vdroplines = NULL;
 }
 
 static void
@@ -1016,10 +1167,31 @@ gog_xy_series_populate_editor (GogObject *obj,
 static void
 gog_xy_series_class_init (GogStyledObjectClass *gso_klass)
 {
+	static GogObjectRole const roles[] = {
+		{ N_("Horizontal drop lines"), "GogSeriesLines", 2,
+			GOG_POSITION_SPECIAL, GOG_POSITION_SPECIAL, GOG_OBJECT_NAME_BY_ROLE,
+			horiz_drop_lines_can_add,
+			NULL,
+			NULL,
+			horiz_drop_lines_post_add,
+			horiz_drop_lines_pre_remove,
+			NULL },
+		{ N_("Vertical drop lines"), "GogSeriesLines",	3,
+			GOG_POSITION_SPECIAL, GOG_POSITION_SPECIAL, GOG_OBJECT_NAME_BY_ROLE,
+			vert_drop_lines_can_add,
+			NULL,
+			NULL,
+			vert_drop_lines_post_add,
+			vert_drop_lines_pre_remove,
+			NULL },
+	};
 	GogObjectClass *gog_klass = (GogObjectClass *)gso_klass;
 	GObjectClass *gobject_klass = (GObjectClass *) gso_klass;
 
 	series_parent_klass = g_type_class_peek_parent (gso_klass);
+	gog_klass->update	= gog_xy_series_update;
+	gog_klass->view_type	= gog_xy_series_view_get_type ();
+	gso_klass->init_style	= gog_xy_series_init_style;
 
 	gobject_klass->finalize		= gog_xy_series_finalize;
 	gobject_klass->set_property = gog_xy_series_set_property;
@@ -1028,6 +1200,8 @@ gog_xy_series_class_init (GogStyledObjectClass *gso_klass)
 	gog_klass->populate_editor	= gog_xy_series_populate_editor;
 	gso_klass->init_style		= gog_xy_series_init_style;
 
+	gog_object_register_roles (gog_klass, roles, G_N_ELEMENTS (roles));
+
 	g_object_class_install_property (gobject_klass, SERIES_PROP_XERRORS,
 		g_param_spec_object ("x-errors", "x-errors",
 			"GogErrorBar *",
@@ -1038,15 +1212,28 @@ gog_xy_series_class_init (GogStyledObjectClass *gso_klass)
 			GOG_ERROR_BAR_TYPE, G_PARAM_READWRITE|GOG_PARAM_PERSISTENT));
 }
 
-static GSF_CLASS (GogXYSeries, gog_xy_series,
-	   gog_xy_series_class_init, gog_xy_series_init,
-	   GOG_SERIES_TYPE)
+static void
+gog_xy_series_base_finalize (GObjectClass *klass)
+{
+	GogObjectClass *go_klass = GOG_OBJECT_CLASS (klass);
+	gog_object_unregister_role (go_klass, "Horizontal drop lines");
+	gog_object_unregister_role (go_klass, "Vertical drop lines");
+}
+
+GSF_DYNAMIC_CLASS_FULL (GogXYSeries, gog_xy_series,
+	NULL, gog_xy_series_base_finalize, gog_xy_series_class_init, NULL,
+	gog_xy_series_init, GOG_SERIES_TYPE, 0, {})
 
 G_MODULE_EXPORT void
 go_plugin_init (GOPlugin *plugin, GOCmdContext *cc)
 {
-	gog_xy_plot_get_type ();
-	gog_bubble_plot_get_type ();
+	GTypeModule *module = go_plugin_get_type_module (plugin);
+	gog_2d_plot_register_type (module);
+	gog_xy_plot_register_type (module);
+	gog_bubble_plot_register_type (module);
+	gog_xy_view_register_type (module);
+	gog_xy_series_view_register_type (module);
+	gog_xy_series_register_type (module);
 }
 
 G_MODULE_EXPORT void
